/****************************************************************************
** $Id: rs_actioninterface.h,v 1.19 2003/09/21 13:17:05 andrew Exp $
**
** Copyright (C) 2001-2003 RibbonSoft. All rights reserved.
**
** This file is part of the qcadlib Library project.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid qcadlib Professional Edition licenses may use 
** this file in accordance with the qcadlib Commercial License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.ribbonsoft.com for further details.
**
** Contact info@ribbonsoft.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/


#ifndef RS_ACTIONINTERFACE_H
#define RS_ACTIONINTERFACE_H

#include <qobject.h>

#include "rs_entitycontainer.h"
#include "rs_commandevent.h"
#include "rs_event.h"
#include "rs_graphic.h"
#include "rs_graphicview.h"
#include "rs_mouseevent.h"
#include "rs_snapper.h"
#include "rs_string.h"
#include "rs_preview.h"
#include "rs_dialogfactory.h"
#include "rs_stringlist.h"

#include "rs_commands.h"

/**
 * This is the interface that must be implemented for all 
 * action classes. Action classes handle actions such
 * as drawing lines, moving entities or zooming in.
 *
 * Inherited from QObject for Qt translation features.
 *
 * @author Andrew Mustun
 */
class RS_ActionInterface : public QObject, public RS_Snapper {
    Q_OBJECT
public:
    /**
     * Constructor.
     *
     * Sets the entity container on which the action class inherited 
     * from this interface operates.
     *
     * @param name Action name. This can be used internally for
     *             debugging mainly.
     * @param container Entity container this action operates on.
     * @param graphicView Graphic view instance this action operates on.
     *                    Please note that an action belongs to this
     *                    view.
     * @param cursor Default mouse cursor for this action. If the action
     *               is suspended and resumed again the cursor will always
     *               be reset to the one given here.
     */
    RS_ActionInterface(const char* name,
                       RS_EntityContainer& container,
                       RS_GraphicView& graphicView) :
    RS_Snapper(container, graphicView) {

        this->name = name;
        status = 0;
        finished = false;
        //triggerOnResume = false;

        // graphic provides a pointer to the graphic if the
        // entity container is a graphic (i.e. can also hold
        // layers).
        graphic = container.getGraphic();

		// document pointer will be used for undo / redo
		document = container.getDocument();

        //this->cursor = cursor;
        //setSnapMode(graphicView.getDefaultSnapMode());


        RS_DEBUG->print("Setting up action: \"%s\"", name);
    }

    /**
     * Destructor.
     */
    virtual ~RS_ActionInterface() {
		//std::cout << "~RS_ActionInterface\n";
		// would be pure virtual now:
		// hideOptions();
		//std::cout << "~RS_ActionInterface: OK\n";
	}

    /**
     * Must be implemented to return the ID of this action.
	 *
	 * @todo no default implementation
     */
    virtual RS2::ActionType rtti()  {
		return RS2::ActionNone;
	}

    /**
     * @return name of this action
     */
    RS_String getName() {
        return name;
    }

    /**
     * Called to initiate an action. This funtcion is often
     * overwritten by the implementing action.
     *
     * @param status The status on which to initiate this action.
     * default is 0 to begin the action.
     */
    virtual void init(int status=0) {
		//std::cout << "init\n";
        RS_Snapper::init();

        setStatus(status);
        if (status>=0) {
            //graphicView->setMouseCursor(cursor);
            updateMouseCursor();
            updateToolBar();
        }
		//std::cout << "init: OK\n";
    }



    /**
     * Called when the mouse moves and this is the current action. 
     * This function can be overwritten by the implementing action.
     * The default implementation keeps track of the mouse position.
     */
    virtual void mouseMoveEvent(RS_MouseEvent*) {}

    /**
     * Called when the left mouse button is pressed and this is the 
     * current action. 
     * This function can be overwritten by the implementing action.
     * The default implementation does nothing.
     */
    virtual void mousePressEvent(RS_MouseEvent*) {}

    /**
     * Called when the left mouse button is released and this is 
     * the current action. 
     * This function can be overwritten by the implementing action.
     * The default implementation does nothing.
     */
    virtual void mouseReleaseEvent(RS_MouseEvent*) {}

    /**
     * Called when a key is pressed and this is the current action.
     * This function can be overwritten by the implementing action.
     * The default implementation does nothing.
     */
    virtual void keyPressEvent(RS_KeyEvent* e) {
		e->ignore();
	}

    /**
     * Called when a key is released and this is the current action.
     * This function can be overwritten by the implementing action.
     * The default implementation does nothing.
     */
    virtual void keyReleaseEvent(RS_KeyEvent* e) {
		e->ignore();
	}

    /**
     * Coordinate event. Triggered usually from a command line.
     * This function can be overwritten by the implementing action.
     * The default implementation does nothing.
     */
    virtual void coordinateEvent(RS_CoordinateEvent*) {}

    /**
     * Called when a command from the command line is launched.
     * and this is the current action.
     * This function can be overwritten by the implementing action.
     * The default implementation does nothing.
     */
    virtual void commandEvent(RS_CommandEvent*) {
        //std::cout << "command: " << e->getCommand().latin1() << "\n";
    }

    /**
     * Must be implemented to return the currently available commands
     *  for the command line.
     */
    virtual RS_StringList getAvailableCommands() {
        RS_StringList l;
        return l;
    }

    /**
     * Sets the current status (progress) of this action.
     * The default implementation sets the class variable 'status' to the
     * given value and finishes the action if 'status' is negative.
     *
     * @param status Status number. It's up to the action implementor
     *               what the action uses the status for. However, a
     *               negative status number finishes the action. Usually
     *               the status of an action increases for every step
     *               of progress and decreases when the user goes one
     *               step back (i.e. presses the right mouse button).
     */
    virtual void setStatus(int status) {
		//std::cout << "setStatus\n";
        this->status = status;
        if (status<0) {
            finish();
            status=0;
        }
        updateMouseButtonHints();
        updateToolBar();
		//std::cout << "setStatus: OK\n";
    }

    /**
     * @return Current status of this action.
     */
    virtual int getStatus() {
        return status;
    }

    /**
     * Triggers this action. This should be called after all
     * data needed for this action was collected / set.
     * The default implementation does nothing.
     */
    virtual void trigger() {}

    /**
     * Should be overwritten to update the mouse button hints
     * wherever they might needed.
     */
    virtual void updateMouseButtonHints() {}

    /**
     * Should be overwritten to set the mouse cursor for this action.
     */
    virtual void updateMouseCursor() {}

    /**
     * Should be overwritten to set the toolbar for this action.
     */
    virtual void updateToolBar() {}

    /**
     * @return true, if the action is finished and can be deleted.
     */
    virtual bool isFinished() {
        return finished;
    }

    /**
     * Finishes this action.
     */
    virtual void finish() {
		//std::cout << "RS_ActionInterface::finish\n";
        status = -1;
        graphicView->setMouseCursor(RS2::ArrowCursor);
        //graphicView->requestToolBar(RS2::ToolBarMain);
        updateToolBar();
        deleteSnapper();
		hideOptions();
        finished = true;
		RS_Snapper::finish();
    }

    /**
     * Called by the event handler to give this action a chance to
     * communicate with its predecessor.
     */
    virtual void setPredecessor(RS_ActionInterface* pre) {
        predecessor = pre;
    }

    /**
     * Suspends this action while another action takes place.
     */
    virtual void suspend() {
        //std::cout << "RS_ActionInterface::suspend()\n";
        graphicView->setMouseCursor(RS2::ArrowCursor);
        RS_Snapper::suspend();
    }

    /**
     * Resumes an action after it was suspended.
     */
    virtual void resume() {
        //std::cout << "RS_ActionInterface::resume()\n";

        //graphicView->setMouseCursor(cursor);
        updateMouseCursor();
		//std::cout << "  001\n";
        updateToolBar();
		//std::cout << "  002\n";
        RS_Snapper::resume();
		
        //std::cout << "RS_ActionInterface::resume() OK\n";
    }

    /**
     * Hides the tool options. Default implementation does nothing.
     */
    virtual void hideOptions() {
        RS_Snapper::hideOptions();
    }

    /**
     * Shows the tool options. Default implementation does nothing.
     */
    virtual void showOptions() {
        RS_Snapper::showOptions();
    }

	/**
	 * Calls checkCommand() from the RS_COMMANDS module.
	 */
    bool checkCommand(const RS_String& cmd, const RS_String& str,
                             RS2::ActionType action=RS2::ActionNone) {
		return RS_COMMANDS->checkCommand(cmd, str, action);
	}

	/**
	 * Calls command() from the RS_COMMANDS module.
	 */
	RS_String command(const RS_String& cmd) {
		return RS_COMMANDS->command(cmd);
	}

	/**
	 * Calls msgAvailableCommands() from the RS_COMMANDS module.
	 */
	RS_String msgAvailableCommands() {
		return RS_COMMANDS->msgAvailableCommands();
	}

private:
    /**
     * Current status of the action. After an action has
     * been created the action status is set to 0. Actions
     * that are terminated have a stats of -1. Other status
     * numbers can be used to describe the stage this action
     * is in. E.g. a window zoom consists of selecting the
     * first corner (status 0), and selecting the second
     * corner (status 1).
     */
    int status;

protected:
    /** Action name. Used internally for debugging */
    RS_String name;

    /**
     * This flag is set when the action has terminated and 
     * can be deleted.
     */
    bool finished;

    /**
     * Pointer to the graphic is this container is a graphic. 
     * NULL otherwise
     */
    RS_Graphic* graphic;

	/**
	 * Pointer to the document (graphic or block) or NULL.
	 */
	RS_Document* document;

    /**
     * Pointer to the default mouse cursor for this action or NULL.
     */
    //RS2::CursorType cursor;

    /**
     * Predecessor of this action or NULL.
     */
    RS_ActionInterface* predecessor;

    /**
     * String prepended to the help text for currently available commands.
     */
    //static RS_String msgAvailableCommands;

    /**
     * Command used for showing help for every action.
     */
    //static RS_String cmdHelp;
	
    /**
     * Command for answering yes to a question.
     */
    //static RS_String cmdYes;
    //static RS_String cmdYes2;
	
     /**
     * Command for answering no to a question.
     */
    //static RS_String cmdNo;
    //static RS_String cmdNo2;
};


#endif
